'
' DotNetNuke - http://www.dotnetnuke.com
' Copyright (c) 2002-2006
' by Perpetual Motion Interactive Systems Inc. ( http://www.perpetualmotion.ca )
'
' Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
' documentation files (the "Software"), to deal in the Software without restriction, including without limitation 
' the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and 
' to permit persons to whom the Software is furnished to do so, subject to the following conditions:
'
' The above copyright notice and this permission notice shall be included in all copies or substantial portions 
' of the Software.
'
' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED 
' TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
' THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
' CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
' DEALINGS IN THE SOFTWARE.
'
Imports DotNetNuke.Services.EventQueue
Namespace DotNetNuke.Entities.Modules
    Public Class EventMessageProcessor
        Inherits EventQueue.EventMessageProcessorBase

        Public Overrides Function ProcessMessage(ByVal message As EventMessage) As Boolean
            Try
                Select Case message.Attributes.Item("ProcessCommand").ToString().ToUpper()


                    Case "UPDATESUPPORTEDFEATURES"
                        UpdateSupportedFeatures(message)
                    Case "UPGRADEMODULE"
                        UpgradeModule(message)
                    Case Else

                End Select
            Catch ex As Exception
                message.ExceptionMessage = ex.Message
                Return False
            End Try
            Return True
        End Function
        Private Sub UpdateSupportedFeatures(ByVal message As EventMessage)
            Dim oObject As Object = Framework.Reflection.CreateObject(message.Attributes.Item("BusinessControllerClass").ToString(), "")
            UpdateSupportedFeatures(oObject, CType(message.Attributes.Item("DesktopModuleId"), Integer))

        End Sub
        Private Sub UpgradeModule(ByVal message As EventMessage)
            Dim BusinessController As String = message.Attributes.Item("BusinessControllerClass").ToString()
            Dim oObject As Object = Framework.Reflection.CreateObject(BusinessController, BusinessController)
            Dim objEventLog As New Services.Log.EventLog.EventLogController
            Dim objEventLogInfo As Services.Log.EventLog.LogInfo
            If TypeOf oObject Is IUpgradeable Then
                ' get the list of applicable versions
                Dim UpgradeVersions As String() = message.Attributes.Item("UpgradeVersionsList").ToString().Split(",".ToCharArray())
                For Each Version As String In UpgradeVersions
                    'call the IUpgradeable interface for the module/version
                    Dim upgradeResults As String = CType(oObject, IUpgradeable).UpgradeModule(Version)
                    'log the upgrade results
                    objEventLogInfo = New Services.Log.EventLog.LogInfo
                    objEventLogInfo.AddProperty("Module Upgraded", BusinessController)
                    objEventLogInfo.AddProperty("Version", Version)
                    objEventLogInfo.AddProperty("Results", upgradeResults)
                    objEventLogInfo.LogTypeKey = Services.Log.EventLog.EventLogController.EventLogType.MODULE_UPDATED.ToString
                    objEventLog.AddLog(objEventLogInfo)
                Next
            End If
            UpdateSupportedFeatures(oObject, CType(message.Attributes.Item("DesktopModuleId"), Integer))
        End Sub
        Private Sub UpdateSupportedFeatures(ByVal oObject As Object, ByVal desktopModuleId As Integer)
            Dim SupportedFeatures As Integer = 0

            If TypeOf oObject Is DotNetNuke.Entities.Modules.IPortable Then
                SupportedFeatures = SupportedFeatures + DesktopModuleSupportedFeature.IsPortable
            End If

            If TypeOf oObject Is DotNetNuke.Entities.Modules.ISearchable Then
                SupportedFeatures = SupportedFeatures + DesktopModuleSupportedFeature.IsSearchable
            End If

            If TypeOf oObject Is DotNetNuke.Entities.Modules.IUpgradeable Then
                SupportedFeatures = SupportedFeatures + DesktopModuleSupportedFeature.IsUpgradeable
            End If

            Dim oDesktopModuleController As New DesktopModuleController
            Dim oDesktopModule As DesktopModuleInfo = oDesktopModuleController.GetDesktopModule(desktopModuleId)
            oDesktopModule.SupportedFeatures = SupportedFeatures
            oDesktopModuleController.UpdateDesktopModule(oDesktopModule)

        End Sub
    End Class
End Namespace
