'
' DotNetNuke - http://www.dotnetnuke.com
' Copyright (c) 2002-2006
' by Perpetual Motion Interactive Systems Inc. ( http://www.perpetualmotion.ca )
'
' Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
' documentation files (the "Software"), to deal in the Software without restriction, including without limitation 
' the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and 
' to permit persons to whom the Software is furnished to do so, subject to the following conditions:
'
' The above copyright notice and this permission notice shall be included in all copies or substantial portions 
' of the Software.
'
' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED 
' TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
' THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
' CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
' DEALINGS IN THE SOFTWARE.
'

Imports System
Imports System.Configuration
Imports System.Data
Imports System.IO

Imports DotNetNuke.Entities.Modules.Actions
Imports DotNetNuke.UI.Skins

Namespace DotNetNuke.Entities.Modules

    ''' -----------------------------------------------------------------------------
    ''' Project	 : DotNetNuke
    ''' Class	 : PortalModuleBase
    ''' 
    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' The PortalModuleBase class defines a custom base class inherited by all
    ''' desktop portal modules within the Portal.
    ''' 
    ''' The PortalModuleBase class defines portal specific properties
    ''' that are used by the portal framework to correctly display portal modules
    ''' </summary>
    ''' <remarks>
    ''' </remarks>
    ''' <history>
    '''		[cnurse]	09/17/2004	Added Documentation
    '''								Modified LocalResourceFile to be Writeable
    '''		[cnurse]	10/21/2004	Modified Settings property to get both
    '''								TabModuleSettings and ModuleSettings
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Public Class PortalModuleBase
        Inherits UserControlBase

#Region "Private Members"

        Private _nextActionId As Integer = -1
        Private _isEditable As Integer = 0
        Private _localResourceFile As String
        Private _moduleConfiguration As ModuleInfo
        Private _settings As Hashtable
        Private _helpfile As String
        Private _helpurl As String
        Private _actions As ModuleActionCollection
        Private _cachedOutput As String = ""

#End Region

#Region "Public Properties"

        <Browsable(False), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)> _
        Public Property Actions() As ModuleActionCollection
            Get
                If _actions Is Nothing Then
                    LoadActions()
                End If
                Return _actions
            End Get
            Set(ByVal Value As ModuleActionCollection)
                _actions = Value
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' The CacheDirectory property is used to return the location of the "Cache"
        ''' Directory for the Module
        ''' </summary>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        '''   [cnurse] 04/28/2005  Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public ReadOnly Property CacheDirectory() As String
            Get
                Return PortalSettings.HomeDirectoryMapPath & "Cache"
            End Get
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' The CacheFileName property is used to store the FileName for this Module's
        ''' Cache
        ''' </summary>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        '''   [cnurse] 04/28/2005  Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public ReadOnly Property CacheFileName() As String
            Get
                Return CacheDirectory & "\" & CleanFileName(CacheKey) & ".htm"
            End Get
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' The CacheKey property is used to calculate a "unique" cache key
        ''' entry to be used to store/retrieve the portal module's content
        ''' from the ASP.NET Cache. Note that cache key allows two versions of the module
        ''' content to be stored - one for anonymous and one for authenticated users.
        ''' </summary>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        '''   [cnurse] 17/9/2004  Added Documentation
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public ReadOnly Property CacheKey() As String
            Get
                Dim strCacheKey As String = "TabModule:"
                strCacheKey += TabModuleId.ToString & ":"
                strCacheKey += System.Threading.Thread.CurrentThread.CurrentCulture.ToString
                Return strCacheKey
            End Get
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' The CacheMethod property is used to store the Method used for this Module's
        ''' Cache
        ''' </summary>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        '''   [cnurse] 04/28/2005  Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public ReadOnly Property CacheMethod() As String
            Get
                Return Convert.ToString(Common.Globals.HostSettings("ModuleCaching"))
            End Get
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' The EditMode property is used to determine whether the user is in the 
        ''' Administrator role
        ''' Cache
        ''' </summary>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        '''   [cnurse] 01/19/2006  Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public ReadOnly Property EditMode() As Boolean
            Get
                Return PortalSecurity.IsInRoles(PortalSettings.AdministratorRoleName) Or PortalSecurity.IsInRoles(PortalSettings.ActiveTab.AdministratorRoles.ToString)
            End Get
        End Property

        <Browsable(False), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)> _
        Public Function EditUrl() As String
            Return EditUrl("", "", "Edit")
        End Function

        <Browsable(False), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)> _
        Public Function EditUrl(ByVal ControlKey As String) As String
            Return EditUrl("", "", ControlKey)
        End Function

        <Browsable(False), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)> _
        Public Function EditUrl(ByVal KeyName As String, ByVal KeyValue As String) As String
            Return EditUrl(KeyName, KeyValue, "Edit")
        End Function

        <Browsable(False), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)> _
        Public Function EditUrl(ByVal KeyName As String, ByVal KeyValue As String, ByVal ControlKey As String) As String

            Dim key As String = ControlKey

            If (key = "") Then
                key = "Edit"
            End If

            If KeyName <> "" And KeyValue <> "" Then
                Return NavigateURL(PortalSettings.ActiveTab.TabID, key, "mid=" & ModuleId.ToString, KeyName & "=" & KeyValue)
            Else
                Return NavigateURL(PortalSettings.ActiveTab.TabID, key, "mid=" & ModuleId.ToString)
            End If

        End Function

        <Browsable(False), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)> _
        Public Function EditUrl(ByVal KeyName As String, ByVal KeyValue As String, ByVal ControlKey As String, ByVal ParamArray AdditionalParameters As String()) As String

            Dim key As String = ControlKey

            If (key = "") Then
                key = "Edit"
            End If

            If KeyName <> "" And KeyValue <> "" Then
                Dim params(AdditionalParameters.Length + 1) As String

                params(0) = "mid=" & ModuleId.ToString
                params(1) = KeyName & "=" & KeyValue

                For i As Integer = 0 To AdditionalParameters.Length - 1
                    params(i + 2) = AdditionalParameters(i)
                Next

                Return NavigateURL(PortalSettings.ActiveTab.TabID, key, params)
            Else
                Dim params(AdditionalParameters.Length) As String

                params(0) = "mid=" & ModuleId.ToString

                For i As Integer = 0 To AdditionalParameters.Length - 1
                    params(i + 1) = AdditionalParameters(i)
                Next

                Return NavigateURL(PortalSettings.ActiveTab.TabID, key, params)
            End If
        End Function

        <ObsoleteAttribute("The HelpFile() property was deprecated in version 2.2. Help files are now stored in the /App_LocalResources folder beneath the module with the following resource key naming convention: ModuleHelp.Text")> _
        Public Property HelpFile() As String
            Get
                Return _helpfile
            End Get
            Set(ByVal Value As String)
                _helpfile = Value
            End Set
        End Property

        Public Property HelpURL() As String
            Get
                Return _helpurl
            End Get
            Set(ByVal Value As String)
                _helpurl = Value
            End Set
        End Property

        <Browsable(False), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)> _
        Public ReadOnly Property IsEditable() As Boolean
            Get

                ' Perform tri-state switch check to avoid having to perform a security
                ' role lookup on every property access (instead caching the result)
                If _isEditable = 0 Then

                    Dim blnPreview As Boolean = False
                    Dim Request As HttpRequest = HttpContext.Current.Request
                    If Not Request.Cookies("_Tab_Admin_Preview" & PortalId.ToString) Is Nothing Then
                        blnPreview = Boolean.Parse(Request.Cookies("_Tab_Admin_Preview" & PortalId.ToString).Value)
                    End If
                    If PortalSettings.ActiveTab.ParentId = PortalSettings.AdminTabId Or PortalSettings.ActiveTab.ParentId = PortalSettings.SuperTabId Then
                        blnPreview = False
                    End If

                    Dim blnHasModuleEditPermissions As Boolean = False
                    If Not _moduleConfiguration Is Nothing Then
                        blnHasModuleEditPermissions = (PortalSecurity.IsInRoles(_moduleConfiguration.AuthorizedEditRoles) = True) Or _
                            (PortalSecurity.IsInRoles(PortalSettings.ActiveTab.AdministratorRoles) = True) Or _
                            (PortalSecurity.IsInRoles(PortalSettings.AdministratorRoleName) = True)
                    End If

                    If blnPreview = False And blnHasModuleEditPermissions = True Then
                        _isEditable = 1
                    Else
                        _isEditable = 2
                    End If
                End If

                Return _isEditable = 1
            End Get
        End Property

        Public Property LocalResourceFile() As String
            Get
                Dim fileRoot As String

                If _localResourceFile = "" Then
                    fileRoot = Me.TemplateSourceDirectory & "/" & Services.Localization.Localization.LocalResourceDirectory & "/" & Me.ID
                Else
                    fileRoot = _localResourceFile
                End If
                Return fileRoot
            End Get
            Set(ByVal Value As String)
                _localResourceFile = Value
            End Set
        End Property

        Public ReadOnly Property ModulePath() As String
            Get
                Return Me.TemplateSourceDirectory & "/"
            End Get
        End Property

        Public ReadOnly Property ControlName() As String
            Get
                Return Me.GetType.Name.Replace("_",".")
            End Get
        End Property

        <Browsable(False), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)> _
        Public Property ModuleConfiguration() As ModuleInfo
            Get
                Return _moduleConfiguration
            End Get
            Set(ByVal Value As ModuleInfo)
                _moduleConfiguration = Value
            End Set
        End Property

        <Browsable(False), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)> _
        Public ReadOnly Property PortalId() As Integer
            Get
                Return PortalSettings.PortalId
            End Get
        End Property

        <Browsable(False), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)> _
        Public ReadOnly Property TabId() As Integer
            Get
                If Not _moduleConfiguration Is Nothing Then
                    Return Convert.ToInt32(_moduleConfiguration.TabID)
                Else
                    Return Null.NullInteger
                End If
            End Get
        End Property

        <Browsable(False), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)> _
        Public ReadOnly Property TabModuleId() As Integer
            Get
                If Not _moduleConfiguration Is Nothing Then
                    Return Convert.ToInt32(_moduleConfiguration.TabModuleID)
                Else
                    Return Null.NullInteger
                End If
            End Get
        End Property

        <Browsable(False), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)> _
        Public ReadOnly Property ModuleId() As Integer
            Get
                If Not _moduleConfiguration Is Nothing Then
                    Return Convert.ToInt32(_moduleConfiguration.ModuleID)
                Else
                    Return Null.NullInteger
                End If
            End Get
        End Property

        <Browsable(False), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)> _
        Public ReadOnly Property UserInfo() As UserInfo
            Get
                Return UserController.GetCurrentUserInfo
            End Get
        End Property

        <Browsable(False), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)> _
        Public ReadOnly Property UserId() As Integer
            Get
                If HttpContext.Current.Request.IsAuthenticated Then
                    UserId = UserInfo.UserID
                Else
                    UserId = -1
                End If
            End Get
        End Property

        <Browsable(False), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)> _
        Public ReadOnly Property PortalAlias() As PortalAliasInfo
            Get
                Return PortalSettings.PortalAlias
            End Get
        End Property

        <Browsable(False), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)> _
        Public ReadOnly Property Settings() As Hashtable
            Get
                If _settings Is Nothing Then
                    _settings = Portals.PortalSettings.GetTabModuleSettings( _
                                    New Hashtable(Portals.PortalSettings.GetModuleSettings(ModuleId)), _
                                    New Hashtable(Portals.PortalSettings.GetTabModuleSettings(TabModuleId)))
                End If
                Return _settings
            End Get
        End Property

        <Browsable(False), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)> _
        Public ReadOnly Property ContainerControl() As Control
            Get
                Return FindControlRecursive(Me, "ctr" & ModuleId.ToString)
            End Get
        End Property

#End Region

#Region "Private Methods"

        Private Function GetActionsCount(ByVal count As Integer, ByVal actions As ModuleActionCollection) As Integer

            For Each action As ModuleAction In actions
                If action.HasChildren Then
                    count += action.Actions.Count

                    'Recursively call to see if this collection has any child actions that would affect the count
                    count = GetActionsCount(count, action.Actions)
                End If
            Next

            Return count

        End Function

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' LoadActions loads the Actions collections
        ''' </summary>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        '''     [cnurse]    01/19/2006  created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Private Sub LoadActions()

            _actions = New ModuleActionCollection

            'check if module Implements Entities.Modules.IActionable interface
            If TypeOf Me Is IActionable Then
                ' load module actions
                Dim ModuleActions As ModuleActionCollection = DirectCast(Me, IActionable).ModuleActions

                _actions.AddRange(ModuleActions)
                _actions.Add(GetNextActionID, "~", "")
                For Each action As ModuleAction In _actions
                    If action.Icon = "" Then
                        action.Icon = "edit.gif"
                    End If
                Next
            End If

            'Make sure the Next Action Id counter is correct
            Dim actionCount As Integer = GetActionsCount(_actions.Count(), _actions)
            If _nextActionId <> actionCount Then
                _nextActionId = actionCount
            End If

            ' check if module implements IPortable interface
            If Me.ModuleConfiguration.IsPortable And Not IsAdminControl() And Me.ModuleConfiguration.BusinessControllerClass <> "" Then
                _actions.Add(GetNextActionID, Services.Localization.Localization.GetString(ModuleActionType.ImportModule, Services.Localization.Localization.GlobalResourceFile), "", "", "rt.gif", NavigateURL(PortalSettings.ActiveTab.TabID, "ImportModule", "moduleid=" & ModuleId.ToString), "", False, SecurityAccessLevel.Admin, EditMode, False)
                _actions.Add(GetNextActionID, Services.Localization.Localization.GetString(ModuleActionType.ExportModule, Services.Localization.Localization.GlobalResourceFile), "", "", "lt.gif", NavigateURL(PortalSettings.ActiveTab.TabID, "ExportModule", "moduleid=" & ModuleId.ToString), "", False, SecurityAccessLevel.Admin, EditMode, False)
            End If

            'If TypeOf objPortalModuleBase Is ISearchable Then
            If Me.ModuleConfiguration.IsSearchable And Not IsAdminControl() And Me.ModuleConfiguration.BusinessControllerClass <> "" Then
                _actions.Add(GetNextActionID, Services.Localization.Localization.GetString(ModuleActionType.SyndicateModule, Services.Localization.Localization.GlobalResourceFile), ModuleActionType.SyndicateModule, "", "xml.gif", NavigateURL(PortalSettings.ActiveTab.TabID, "", "moduleid=" & ModuleId.ToString).Replace(glbDefaultPage, "RSS.aspx"), "", False, SecurityAccessLevel.Anonymous, True, True)
            End If

            ' help module actions available to content editors and administrators
            If ModuleConfiguration.ControlType <> SecurityAccessLevel.Anonymous And Request.QueryString("ctl") <> "Help" Then
                SetHelpVisibility()
            End If

            'Add Print Action
            If ModuleConfiguration.DisplayPrint Then
                ' print module action available to everyone
                _actions.Add(GetNextActionID, Services.Localization.Localization.GetString(ModuleActionType.PrintModule, Services.Localization.Localization.GlobalResourceFile), ModuleActionType.PrintModule, "", "print.gif", NavigateURL(TabId, "", "mid=" & ModuleId.ToString, "SkinSrc=" & QueryStringEncode("[G]" & SkinInfo.RootSkin & "/" & glbHostSkinFolder & "/" & "No Skin"), "ContainerSrc=" & QueryStringEncode("[G]" & SkinInfo.RootContainer & "/" & glbHostSkinFolder & "/" & "No Container"), "dnnprintmode=true"), "", False, SecurityAccessLevel.Anonymous, True, True)
            End If

            ' core module actions only available to administrators 
            If EditMode = True And ModuleConfiguration.IsAdmin = False And IsAdminControl() = False Then
                ' module settings
                _actions.Add(GetNextActionID, "~", "")
                _actions.Add(GetNextActionID, Services.Localization.Localization.GetString(ModuleActionType.ModuleSettings, Services.Localization.Localization.GlobalResourceFile), ModuleActionType.ModuleSettings, "", "settings.gif", NavigateURL(TabId, "Module", "ModuleId=" & ModuleId.ToString), secure:=SecurityAccessLevel.Admin, Visible:=True)
                _actions.Add(GetNextActionID, Services.Localization.Localization.GetString(ModuleActionType.DeleteModule, Services.Localization.Localization.GlobalResourceFile), ModuleActionType.DeleteModule, ModuleConfiguration.ModuleID.ToString, "delete.gif", "", "confirm('" + DotNetNuke.UI.Utilities.ClientAPI.GetSafeJSString(Services.Localization.Localization.GetString("DeleteModule.Confirm")) + "')", False, SecurityAccessLevel.Admin, True, False)
                If ModuleConfiguration.CacheTime <> 0 Then
                    _actions.Add(GetNextActionID, Services.Localization.Localization.GetString(ModuleActionType.ClearCache, Services.Localization.Localization.GlobalResourceFile), ModuleActionType.ClearCache, ModuleConfiguration.ModuleID.ToString, "restore.gif", secure:=SecurityAccessLevel.Admin, Visible:=True)
                End If

                ' module movement
                _actions.Add(GetNextActionID, "~", "")
                Dim MoveActionRoot As New ModuleAction(GetNextActionID, Services.Localization.Localization.GetString(ModuleActionType.MoveRoot, Services.Localization.Localization.GlobalResourceFile), "", "", "", "", "", False, SecurityAccessLevel.Admin, EditMode)

                ' move module up/down
                If Not ModuleConfiguration Is Nothing Then
                    SetMoveMenuVisibility(MoveActionRoot.Actions.Add(GetNextActionID(), Services.Localization.Localization.GetString(ModuleActionType.MoveTop, Services.Localization.Localization.GlobalResourceFile), ModuleActionType.MoveTop, ModuleConfiguration.PaneName, Icon:="top.gif", secure:=SecurityAccessLevel.Admin, Visible:=EditMode))
                    SetMoveMenuVisibility(MoveActionRoot.Actions.Add(GetNextActionID(), Services.Localization.Localization.GetString(ModuleActionType.MoveUp, Services.Localization.Localization.GlobalResourceFile), ModuleActionType.MoveUp, ModuleConfiguration.PaneName, Icon:="up.gif", secure:=SecurityAccessLevel.Admin, Visible:=EditMode))
                    SetMoveMenuVisibility(MoveActionRoot.Actions.Add(GetNextActionID(), Services.Localization.Localization.GetString(ModuleActionType.MoveDown, Services.Localization.Localization.GlobalResourceFile), ModuleActionType.MoveDown, ModuleConfiguration.PaneName, Icon:="dn.gif", secure:=SecurityAccessLevel.Admin, Visible:=EditMode))
                    SetMoveMenuVisibility(MoveActionRoot.Actions.Add(GetNextActionID(), Services.Localization.Localization.GetString(ModuleActionType.MoveBottom, Services.Localization.Localization.GlobalResourceFile), ModuleActionType.MoveBottom, ModuleConfiguration.PaneName, Icon:="bottom.gif", secure:=SecurityAccessLevel.Admin, Visible:=EditMode))
                End If

                ' move module to pane
                Dim intItem As Integer
                For intItem = 0 To PortalSettings.ActiveTab.Panes.Count - 1
                    SetMoveMenuVisibility(MoveActionRoot.Actions.Add(GetNextActionID(), Services.Localization.Localization.GetString(ModuleActionType.MovePane, Services.Localization.Localization.GlobalResourceFile) & " " & Convert.ToString(PortalSettings.ActiveTab.Panes(intItem)), ModuleActionType.MovePane, Convert.ToString(PortalSettings.ActiveTab.Panes(intItem)), Icon:="move.gif", secure:=SecurityAccessLevel.Admin, Visible:=EditMode))
                Next intItem
                Dim ma As ModuleAction
                For Each ma In MoveActionRoot.Actions
                    If ma.Visible Then
                        _actions.Add(MoveActionRoot)
                        Exit For
                    End If
                Next
            End If
        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' SetHelpVisibility Adds the Help actions to the Action Menu
        ''' </summary>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[cnurse]	05/12/2005	Documented
        '''     [cnurse]    01/19/2006  Moved from ActionBase
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Private Sub SetHelpVisibility()

            'Add Help Menu Action
            Dim helpAction As New ModuleAction(GetNextActionID)
            helpAction.Title = Services.Localization.Localization.GetString(ModuleActionType.ModuleHelp, Services.Localization.Localization.GlobalResourceFile)
            helpAction.CommandName = ModuleActionType.ModuleHelp
            helpAction.CommandArgument = ""
            helpAction.Icon = "help.gif"
            helpAction.Url = NavigateURL(TabId, "Help", "ctlid=" & ModuleConfiguration.ModuleControlId.ToString, "moduleid=" & ModuleId)
            helpAction.Secure = SecurityAccessLevel.Edit
            helpAction.Visible = True
            helpAction.NewWindow = False
            helpAction.UseActionEvent = True
            _actions.Add(helpAction)


            'Add OnLine Help Action
            Dim helpUrl As String = GetOnLineHelp(ModuleConfiguration.HelpUrl, ModuleConfiguration)
            If Not Null.IsNull(helpUrl) Then
                'Add OnLine Help menu action
                helpAction = New ModuleAction(GetNextActionID)
                helpAction.Title = Services.Localization.Localization.GetString(ModuleActionType.OnlineHelp, Services.Localization.Localization.GlobalResourceFile)
                helpAction.CommandName = ModuleActionType.OnlineHelp
                helpAction.CommandArgument = ""
                helpAction.Icon = "help.gif"
                helpAction.Url = FormatHelpUrl(helpUrl, PortalSettings, ModuleConfiguration.FriendlyName)
                helpAction.Secure = SecurityAccessLevel.Edit
                helpAction.UseActionEvent = True
                helpAction.Visible = True
                helpAction.NewWindow = True
                _actions.Add(helpAction)
            End If

        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' SetMoveMenuVisibility Adds the Move actions to the Action Menu
        ''' </summary>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[cnurse]	05/12/2005	Documented
        '''     [cnurse]    01/19/2006  Moved from ActionBase
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Private Sub SetMoveMenuVisibility(ByVal Action As ModuleAction)

            Select Case Action.CommandName
                Case ModuleActionType.MoveTop
                    Action.Visible = (ModuleConfiguration.ModuleOrder <> 0) And (ModuleConfiguration.PaneModuleIndex > 0) And EditMode
                Case ModuleActionType.MoveUp
                    Action.Visible = (ModuleConfiguration.ModuleOrder <> 0) And (ModuleConfiguration.PaneModuleIndex > 0) And EditMode
                Case ModuleActionType.MoveDown
                    Action.Visible = (ModuleConfiguration.ModuleOrder <> 0) And (ModuleConfiguration.PaneModuleIndex < (ModuleConfiguration.PaneModuleCount - 1)) And EditMode
                Case ModuleActionType.MoveBottom
                    Action.Visible = (ModuleConfiguration.ModuleOrder <> 0) And (ModuleConfiguration.PaneModuleIndex < (ModuleConfiguration.PaneModuleCount - 1)) And EditMode
                Case ModuleActionType.MovePane
                    Action.Visible = (LCase(ModuleConfiguration.PaneName) <> LCase(Action.CommandArgument)) And EditMode
            End Select

        End Sub

#End Region

#Region "Protected Methods"

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Helper method that can be used to add an ActionEventHandler to the Skin for this 
        ''' Module Control
        ''' </summary>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        '''   [cnurse] 17/9/2004  Added Documentation
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Protected Sub AddActionHandler(ByVal e As ActionEventHandler)

            'This finds a reference to the containing skin
            Dim ParentSkin As DotNetNuke.UI.Skins.Skin = DotNetNuke.UI.Skins.Skin.GetParentSkin(Me)
            'We should always have a ParentSkin, but need to make sure
            If Not ParentSkin Is Nothing Then
                'Register our EventHandler as a listener on the ParentSkin so that it may tell us 
                'when a menu has been clicked.
                ParentSkin.RegisterModuleActionEvent(Me.ModuleId, e)
            End If


        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' The CreateChildControls method is called when the ASP.NET Page Framework
        ''' determines that it is time to instantiate a server control.
        ''' This method and attempts to resolve any previously cached output of the portal 
        ''' module from the ASP.NET cache.  
        ''' If it doesn't find cached output from a previous request, then it will instantiate 
        ''' and add the portal modules UserControl instance into the page tree.
        ''' </summary>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        '''   [cnurse] 17/9/2004  Added Documentation
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Protected Overrides Sub CreateChildControls()

            If Not _moduleConfiguration Is Nothing Then

                ' if user does not have EDIT rights for the module ( content editors can not see cached versions of modules )
                If PortalSecurity.HasEditPermissions(_moduleConfiguration.ModuleID) = False Then

                    ' Attempt to resolve previously cached content 
                    If _moduleConfiguration.CacheTime <> 0 Then
                        If CacheMethod <> "D" Then
                            _cachedOutput = Convert.ToString(DataCache.GetCache(CacheKey))
                        Else ' cache from disk
                            If File.Exists(CacheFileName) Then
                                Dim cacheFile As New FileInfo(CacheFileName)
                                If cacheFile.CreationTime.AddSeconds(_moduleConfiguration.CacheTime) >= Now Then
                                    'Load from Cache
                                    Dim objStreamReader As StreamReader
                                    objStreamReader = cacheFile.OpenText()
                                    _cachedOutput = objStreamReader.ReadToEnd
                                    objStreamReader.Close()
                                Else
                                    'Cache Expired so delete it
                                    cacheFile.Delete()
                                End If
                            End If
                        End If
                    End If

                    ' If no cached content is found, then instantiate and add the portal
                    ' module user control into the portal's page server control tree
                    If _cachedOutput = "" And _moduleConfiguration.CacheTime > 0 Then

                        MyBase.CreateChildControls()

                        Dim objPortalModuleBase As Entities.Modules.PortalModuleBase = CType(Page.LoadControl(_moduleConfiguration.ControlSrc), PortalModuleBase)
                        objPortalModuleBase.ModuleConfiguration = Me.ModuleConfiguration

                        ' In skin.vb, the call to Me.Controls.Add(objPortalModuleBase) calls CreateChildControls() therefore
                        ' we need to indicate the control has already been created. We will manipulate the CacheTime property for this purpose. 
                        objPortalModuleBase.ModuleConfiguration.CacheTime = -(objPortalModuleBase.ModuleConfiguration.CacheTime)

                        Me.Controls.Add(objPortalModuleBase)

                    Else
                        ' restore the CacheTime property in preparation for the Render() event
                        If _moduleConfiguration.CacheTime < 0 Then
                            _moduleConfiguration.CacheTime = -(_moduleConfiguration.CacheTime)
                        End If
                    End If

                End If
            End If

        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' The Render method is called when the ASP.NET Page Framework
        ''' determines that it is time to render content into the page output stream.
        ''' This method and captures the output generated by the portal module user control
        ''' It then adds this content into the ASP.NET Cache for future requests.
        ''' </summary>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        '''   [cnurse] 17/9/2004  Added Documentation
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Protected Overrides Sub Render(ByVal output As HtmlTextWriter)

            If Not _moduleConfiguration Is Nothing Then

                ' If no caching is specified or in admin mode, render the child tree and return 
                If _moduleConfiguration.CacheTime = 0 Or (PortalSecurity.IsInRoles(PortalSettings.AdministratorRoleName) = True Or PortalSecurity.IsInRoles(PortalSettings.ActiveTab.AdministratorRoles.ToString) = True) Then

                    MyBase.Render(output)

                Else 'output caching enabled

                    ' If no cached output was found from a previous request, render
                    ' child controls into a TextWriter, and then cache the results
                    If _cachedOutput = "" Then

                        Dim tempWriter As StringWriter = New StringWriter
                        MyBase.Render(New HtmlTextWriter(tempWriter))
                        _cachedOutput = tempWriter.ToString()

                        If CacheMethod <> "D" Then
                            DataCache.SetCache(CacheKey, _cachedOutput, DateTime.Now.AddSeconds(_moduleConfiguration.CacheTime))
                        Else ' cache to disk
                            Try
                                If Not Directory.Exists(CacheDirectory) Then
                                    Directory.CreateDirectory(CacheDirectory)
                                End If

                                Dim objStream As StreamWriter
                                objStream = File.CreateText(CacheFileName)
                                objStream.WriteLine(_cachedOutput)
                                objStream.Close()
                            Catch
                                ' error writing to disk
                            End Try
                        End If
                    End If

                    ' Output the user control's content
                    output.Write(_cachedOutput)
                End If

            Else

                MyBase.Render(output)

            End If

        End Sub

#End Region

#Region "Public Methods"

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Gets the Next Action ID
        ''' </summary>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        '''   [cnurse] 03/02/2006  Added Documentation
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public Function GetNextActionID() As Integer
            _nextActionId += 1
            Return _nextActionId
        End Function

        Public Sub SynchronizeModule()
            If CacheMethod <> "D" Then
                DataCache.RemoveCache(CacheKey)
            Else ' disk caching
                If System.IO.File.Exists(CacheFileName) Then
                    Try
                        System.IO.File.Delete(CacheFileName)
                    Catch
                        ' error deleting file from disk
                    End Try
                End If
            End If
        End Sub

        Public Function HasModulePermission(ByVal PermissionKey As String) As Boolean
            Return Security.Permissions.ModulePermissionController.HasModulePermission(Me.ModuleConfiguration.ModulePermissions, PermissionKey)
        End Function

#End Region

    End Class

End Namespace
