'
' DotNetNuke - http://www.dotnetnuke.com
' Copyright (c) 2002-2006
' by Perpetual Motion Interactive Systems Inc. ( http://www.perpetualmotion.ca )
'
' Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
' documentation files (the "Software"), to deal in the Software without restriction, including without limitation 
' the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and 
' to permit persons to whom the Software is furnished to do so, subject to the following conditions:
'
' The above copyright notice and this permission notice shall be included in all copies or substantial portions 
' of the Software.
'
' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED 
' TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
' THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
' CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
' DEALINGS IN THE SOFTWARE.
'
Imports System
Imports System.Collections
Imports System.Configuration
Imports System.Diagnostics
Imports System.IO
Imports System.Reflection
Imports System.Web.Configuration
Imports System.Xml

Imports DotNetNuke.Framework.Providers

Namespace DotNetNuke.Common.Utilities

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' The Config class provides access to the web.config file
    ''' </summary>
    ''' <remarks>
    ''' </remarks>
    ''' <history>
    '''		[cnurse]	11/15/2005	documented
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Public Class Config

#Region "Shared Methods"

        Public Shared Sub AddCodeSubDirectory(ByVal name As String)

            Dim webConfig As System.Configuration.Configuration = WebConfigurationManager.OpenWebConfiguration(Globals.ApplicationPath)

            Dim configSection As CompilationSection = CType(webConfig.GetSection("system.web/compilation"), CompilationSection)
            Dim directoryFound As Boolean = False
            For Each codeSubDirectory As CodeSubDirectory In configSection.CodeSubDirectories
                If codeSubDirectory.DirectoryName = name Then
                    directoryFound = True
                    Exit For
                End If
            Next

            If Not directoryFound Then
                configSection.CodeSubDirectories.Add(New CodeSubDirectory(name))
                webConfig.Save()
            End If

        End Sub

        Public Shared Function AddAppSetting(ByVal xmlDoc As XmlDocument, ByVal Key As String, ByVal Value As String) As XmlDocument

            Dim xmlElement As XmlElement

            ' retrieve the appSettings node 
            Dim xmlAppSettings As XmlNode = xmlDoc.SelectSingleNode("//appSettings")

            If Not xmlAppSettings Is Nothing Then
                ' get the node based on key
                Dim xmlNode As XmlNode = xmlAppSettings.SelectSingleNode(("//add[@key='" + Key + "']"))

                If Not xmlNode Is Nothing Then
                    ' update the existing element
                    xmlElement = CType(xmlNode, XmlElement)
                    xmlElement.SetAttribute("value", Value)
                Else
                    ' create a new element
                    xmlElement = xmlDoc.CreateElement("add")
                    xmlElement.SetAttribute("key", Key)
                    xmlElement.SetAttribute("value", Value)
                    xmlAppSettings.AppendChild(xmlElement)
                End If
            End If

            ' return the xml doc
            Return xmlDoc

        End Function

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Gets the default connection String as specified in the provider.
        ''' </summary>
        ''' <returns>The connection String</returns>
        ''' <remarks></remarks>
        ''' <history>
        '''		[cnurse]	11/15/2005	created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public Shared Function GetConnectionString() As String
            Dim _providerConfiguration As ProviderConfiguration = ProviderConfiguration.GetProviderConfiguration("data")

            ' Read the configuration specific information for this provider
            Dim objProvider As Provider = CType(_providerConfiguration.Providers(_providerConfiguration.DefaultProvider), Provider)

            Return GetConnectionString(objProvider.Attributes("connectionStringName"))
        End Function

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Gets the specified connection String
        ''' </summary>
        ''' <param name="name">Name of Connection String to return</param>
        ''' <returns>The connection String</returns>
        ''' <remarks></remarks>
        ''' <history>
        '''		[cnurse]	11/15/2005	created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public Shared Function GetConnectionString(ByVal name As String) As String

            Dim connectionString As String = ""

            'First check if connection string is specified in <connectionstrings> (ASP.NET 2.0 / DNN v4.x)
            If name <> "" Then
                'ASP.NET 2 version connection string (in <connectionstrings>)
                'This will be for new v4.x installs or upgrades from v4.x
                connectionString = WebConfigurationManager.ConnectionStrings(name).ConnectionString
            End If

            If connectionString = "" Then
                'Next check if connection string is specified in <appsettings> (ASP.NET 1.1 / DNN v3.x)
                'This will accomodate upgrades from v3.x
                If name <> "" Then
                    connectionString = Config.GetSetting(name)
                End If
            End If

            Return connectionString
        End Function

        Public Shared Function GetSetting(ByVal setting As String) As String
            Return WebConfigurationManager.AppSettings(setting)
        End Function

        Public Shared Function GetSection(ByVal section As String) As Object
            Return WebConfigurationManager.GetSection(section)
        End Function

        Public Shared Function Load() As XmlDocument
            ' open the web.config file
            Dim xmlDoc As New XmlDocument
            xmlDoc.Load(Common.Globals.ApplicationMapPath & "\web.config")
            Return xmlDoc
        End Function

        Public Shared Function Load(ByVal filename As String) As XmlDocument
            ' open the config file
            Dim xmlDoc As New XmlDocument
            xmlDoc.Load(Common.Globals.ApplicationMapPath & "\" & filename)
            Return xmlDoc
        End Function

        Public Shared Sub RemoveCodeSubDirectory(ByVal name As String)
            Dim webConfig As System.Configuration.Configuration = WebConfigurationManager.OpenWebConfiguration(Globals.ApplicationPath)

            Dim configSection As CompilationSection = CType(webConfig.GetSection("system.web/compilation"), CompilationSection)
            Dim directoryFound As Boolean = False
            For Each codeSubDirectory As CodeSubDirectory In configSection.CodeSubDirectories
                If codeSubDirectory.DirectoryName = name Then
                    directoryFound = True
                    Exit For
                End If
            Next

            If directoryFound Then
                configSection.CodeSubDirectories.Remove(name)
                webConfig.Save()
            End If

        End Sub

        Public Shared Function Save(ByVal xmlDoc As XmlDocument) As String
            Try
                ' save the config file
                Dim writer As New XmlTextWriter(Common.Globals.ApplicationMapPath & "\web.config", Nothing)
                writer.Formatting = Formatting.Indented
                xmlDoc.WriteTo(writer)
                writer.Flush()
                writer.Close()
                Return ""
            Catch exc As Exception
                ' the file may be read-only or the file permissions may not be set properly
                Return exc.Message
            End Try

        End Function

        Public Shared Function Save(ByVal xmlDoc As XmlDocument, ByVal filename As String) As String
            Try
                ' save the config file
                Dim writer As New XmlTextWriter(Common.Globals.ApplicationMapPath & "\" & filename, Nothing)
                writer.Formatting = Formatting.Indented
                xmlDoc.WriteTo(writer)
                writer.Flush()
                writer.Close()
                Return ""
            Catch exc As Exception
                ' the file may be read-only or the file permissions may not be set properly
                Return exc.Message
            End Try

        End Function

        Public Shared Function Touch() As Boolean
            File.SetLastWriteTime(Common.Globals.ApplicationMapPath & "\web.config", System.DateTime.Now)
        End Function

        Public Shared Function UpdateMachineKey(ByVal xmlConfig As XmlDocument) As XmlDocument

            Dim objSecurity As New PortalSecurity
            Dim validationKey As String = objSecurity.CreateKey(20)
            Dim decryptionKey As String = objSecurity.CreateKey(24)

            Dim xmlMachineKey As XmlNode = xmlConfig.SelectSingleNode("configuration/system.web/machineKey")

            xmlMachineKey.Attributes("validationKey").InnerText = validationKey
            xmlMachineKey.Attributes("decryptionKey").InnerText = decryptionKey

            xmlConfig = AddAppSetting(xmlConfig, "InstallationDate", Date.Today.ToShortDateString)

            Return xmlConfig

        End Function


#End Region

    End Class

End Namespace
