'
' DotNetNuke - http://www.dotnetnuke.com
' Copyright (c) 2002-2005
' by Perpetual Motion Interactive Systems Inc. ( http://www.perpetualmotion.ca )
'
' Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
' documentation files (the "Software"), to deal in the Software without restriction, including without limitation 
' the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and 
' to permit persons to whom the Software is furnished to do so, subject to the following conditions:
'
' The above copyright notice and this permission notice shall be included in all copies or substantial portions 
' of the Software.
'
' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED 
' TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
' THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
' CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
' DEALINGS IN THE SOFTWARE.
'

Imports System
Imports System.Data
Imports System.Data.SqlClient
Imports Microsoft.ApplicationBlocks.Data
Imports DotNetNuke


Namespace DotNetNuke.Security.Membership.Data

    Public Class SqlDataProvider
        Inherits DataProvider

#Region "Private Members"
        Private Const ProviderType As String = "data"
        Private _providerConfiguration As Framework.Providers.ProviderConfiguration = Framework.Providers.ProviderConfiguration.GetProviderConfiguration(ProviderType)
        Private _connectionString As String
        Private _providerPath As String
        Private _objectQualifier As String
        Private _databaseOwner As String
#End Region

#Region "Constructors"

        Public Sub New()

            ' Read the configuration specific information for this provider
            Dim objProvider As Framework.Providers.Provider = CType(_providerConfiguration.Providers(_providerConfiguration.DefaultProvider), Framework.Providers.Provider)

            ' Read the attributes for this provider
            'Get Connection string from web.config
            _connectionString = Config.GetConnectionString()

            If _connectionString = "" Then
                ' Use connection string specified in provider
                _connectionString = objProvider.Attributes("connectionString")
            End If

            _providerPath = objProvider.Attributes("providerPath")

            _objectQualifier = objProvider.Attributes("objectQualifier")
            If _objectQualifier <> "" And _objectQualifier.EndsWith("_") = False Then
                _objectQualifier += "_"
            End If

            _databaseOwner = objProvider.Attributes("databaseOwner")
            If _databaseOwner <> "" And _databaseOwner.EndsWith(".") = False Then
                _databaseOwner += "."
            End If

        End Sub

#End Region

#Region "Properties"

        Public ReadOnly Property ConnectionString() As String
            Get
                Return _connectionString
            End Get
        End Property

        Public ReadOnly Property ProviderPath() As String
            Get
                Return _providerPath
            End Get
        End Property

        Public ReadOnly Property ObjectQualifier() As String
            Get
                Return _objectQualifier
            End Get
        End Property

        Public ReadOnly Property DatabaseOwner() As String
            Get
                Return _databaseOwner
            End Get
        End Property

#End Region

#Region "General Public Methods"

        Private Function GetNull(ByVal Field As Object) As Object
            Return DotNetNuke.Common.Utilities.Null.GetNull(Field, DBNull.Value)
        End Function

#End Region

#Region "Public Methods"

        'Security
        Public Overrides Function UserLogin(ByVal Username As String, ByVal Password As String) As IDataReader
            Return CType(SqlHelper.ExecuteReader(ConnectionString, DatabaseOwner & ObjectQualifier & "UserLogin", Username, Password), IDataReader)
        End Function

        Public Overrides Function GetAuthRoles(ByVal PortalId As Integer, ByVal ModuleId As Integer) As IDataReader
            Return CType(SqlHelper.ExecuteReader(ConnectionString, DatabaseOwner & ObjectQualifier & "GetAuthRoles", PortalId, ModuleId), IDataReader)
        End Function

        'Users
        Public Overrides Function AddUser(ByVal PortalID As Integer, ByVal Username As String, ByVal FirstName As String, ByVal LastName As String, ByVal AffiliateId As Integer, ByVal IsSuperUser As Boolean, ByVal Email As String) As Integer
            Try
                Return CType(SqlHelper.ExecuteScalar(ConnectionString, DatabaseOwner & ObjectQualifier & "AddUser", PortalID, Username, FirstName, LastName, GetNull(AffiliateId), IsSuperUser, Email), Integer)
            Catch    ' duplicate
                Return -1
            End Try
        End Function

        Public Overrides Sub DeleteUserPortal(ByVal UserId As Integer, ByVal PortalId As Integer)
            SqlHelper.ExecuteNonQuery(ConnectionString, DatabaseOwner & ObjectQualifier & "DeleteUserPortal", UserId, PortalId)
        End Sub

        Public Overrides Sub DeleteUser(ByVal UserId As Integer)
            SqlHelper.ExecuteNonQuery(ConnectionString, DatabaseOwner & ObjectQualifier & "DeleteUser", UserId)
        End Sub

        Public Overrides Sub UpdateUser(ByVal UserId As Integer, ByVal FirstName As String, ByVal LastName As String, ByVal Email As String)
            SqlHelper.ExecuteNonQuery(ConnectionString, DatabaseOwner & ObjectQualifier & "UpdateUser", UserId, FirstName, LastName, Email)
        End Sub

        Public Overrides Function GetUser(ByVal PortalId As Integer, ByVal UserId As Integer) As IDataReader
            Return CType(SqlHelper.ExecuteReader(ConnectionString, DatabaseOwner & ObjectQualifier & "GetUser", PortalId, UserId), IDataReader)
        End Function

        Public Overrides Function GetUserByUsername(ByVal PortalId As Integer, ByVal Username As String) As IDataReader
            Return CType(SqlHelper.ExecuteReader(ConnectionString, DatabaseOwner & ObjectQualifier & "GetUserByUsername", GetNull(PortalId), Username), IDataReader)
        End Function

        Public Overrides Function GetSuperUsers() As IDataReader
            Return CType(SqlHelper.ExecuteReader(ConnectionString, DatabaseOwner & ObjectQualifier & "GetSuperUsers"), IDataReader)
        End Function

        'Roles
        Public Overrides Function GetRolesByUser(ByVal UserId As Integer, ByVal PortalId As Integer) As IDataReader
            Return CType(SqlHelper.ExecuteReader(ConnectionString, DatabaseOwner & ObjectQualifier & "GetRolesByUser", UserId, PortalId), IDataReader)
        End Function

        Public Overrides Function GetPortalRoles(ByVal PortalId As Integer) As IDataReader
            Return CType(SqlHelper.ExecuteReader(ConnectionString, DatabaseOwner & ObjectQualifier & "GetPortalRoles", PortalId), IDataReader)
        End Function

        Public Overrides Function GetRoles() As IDataReader
            Return CType(SqlHelper.ExecuteReader(ConnectionString, DatabaseOwner & ObjectQualifier & "GetRoles"), IDataReader)
        End Function

        Public Overrides Function GetRole(ByVal RoleId As Integer, ByVal PortalId As Integer) As IDataReader
            Return CType(SqlHelper.ExecuteReader(ConnectionString, DatabaseOwner & ObjectQualifier & "GetRole", RoleId, PortalId), IDataReader)
        End Function

        Public Overrides Function GetRoleByName(ByVal PortalId As Integer, ByVal RoleName As String) As IDataReader
            Return CType(SqlHelper.ExecuteReader(ConnectionString, DatabaseOwner & ObjectQualifier & "GetRoleByName", PortalId, RoleName), IDataReader)
        End Function

        Public Overrides Function GetRolesByGroup(ByVal RoleGroupId As Integer, ByVal PortalId As Integer) As IDataReader
            Return CType(SqlHelper.ExecuteReader(ConnectionString, DatabaseOwner & ObjectQualifier & "GetRolesByGroup", GetNull(RoleGroupId), PortalId), IDataReader)
        End Function

        Public Overrides Function AddRole(ByVal PortalId As Integer, ByVal RoleGroupId As Integer, ByVal RoleName As String, ByVal Description As String, ByVal ServiceFee As Single, ByVal BillingPeriod As String, ByVal BillingFrequency As String, ByVal TrialFee As Single, ByVal TrialPeriod As Integer, ByVal TrialFrequency As String, ByVal IsPublic As Boolean, ByVal AutoAssignment As Boolean, ByVal RSVPCode As String) As Integer
            Return CType(SqlHelper.ExecuteScalar(ConnectionString, DatabaseOwner & ObjectQualifier & "AddRole", PortalId, GetNull(RoleGroupId), RoleName, Description, ServiceFee, BillingPeriod, GetNull(BillingFrequency), TrialFee, TrialPeriod, GetNull(TrialFrequency), IsPublic, AutoAssignment, RSVPCode), Integer)
        End Function

        Public Overrides Sub DeleteRole(ByVal RoleId As Integer)
            SqlHelper.ExecuteNonQuery(ConnectionString, DatabaseOwner & ObjectQualifier & "DeleteRole", RoleId)
        End Sub

        Public Overrides Sub UpdateRole(ByVal RoleId As Integer, ByVal RoleGroupId As Integer, ByVal Description As String, ByVal ServiceFee As Single, ByVal BillingPeriod As String, ByVal BillingFrequency As String, ByVal TrialFee As Single, ByVal TrialPeriod As Integer, ByVal TrialFrequency As String, ByVal IsPublic As Boolean, ByVal AutoAssignment As Boolean, ByVal RSVPCode As String)
            SqlHelper.ExecuteNonQuery(ConnectionString, DatabaseOwner & ObjectQualifier & "UpdateRole", RoleId, GetNull(RoleGroupId), Description, ServiceFee, BillingPeriod, GetNull(BillingFrequency), TrialFee, TrialPeriod, GetNull(TrialFrequency), IsPublic, AutoAssignment, RSVPCode)
        End Sub

        'Role Groups
        Public Overrides Function AddRoleGroup(ByVal PortalId As Integer, ByVal GroupName As String, ByVal Description As String) As Integer
            Return CType(SqlHelper.ExecuteScalar(ConnectionString, DatabaseOwner & ObjectQualifier & "AddRoleGroup", PortalId, GroupName, Description), Integer)
        End Function

        Public Overrides Sub DeleteRoleGroup(ByVal RoleGroupId As Integer)
            SqlHelper.ExecuteNonQuery(ConnectionString, DatabaseOwner & ObjectQualifier & "DeleteRoleGroup", RoleGroupId)
        End Sub

        Public Overrides Function GetRoleGroup(ByVal portalId As Integer, ByVal roleGroupId As Integer) As IDataReader
            Return CType(SqlHelper.ExecuteReader(ConnectionString, DatabaseOwner & ObjectQualifier & "GetRoleGroup", portalId, roleGroupId), IDataReader)
        End Function

        Public Overrides Function GetRoleGroups(ByVal portalId As Integer) As IDataReader
            Return CType(SqlHelper.ExecuteReader(ConnectionString, DatabaseOwner & ObjectQualifier & "GetRoleGroups", portalId), IDataReader)
        End Function

        Public Overrides Sub UpdateRoleGroup(ByVal RoleGroupId As Integer, ByVal GroupName As String, ByVal Description As String)
            SqlHelper.ExecuteNonQuery(ConnectionString, DatabaseOwner & ObjectQualifier & "UpdateRoleGroup", RoleGroupId, GroupName, Description)
        End Sub

        'User Roles
        Public Overrides Function GetUserRole(ByVal PortalId As Integer, ByVal UserId As Integer, ByVal RoleId As Integer) As IDataReader
            Return CType(SqlHelper.ExecuteReader(ConnectionString, DatabaseOwner & ObjectQualifier & "GetUserRole", PortalId, UserId, RoleId), IDataReader)
        End Function

        Public Overrides Function GetUserRolesByUsername(ByVal PortalID As Integer, ByVal Username As String, ByVal Rolename As String) As IDataReader
            Return CType(SqlHelper.ExecuteReader(ConnectionString, DatabaseOwner & ObjectQualifier & "GetUserRolesByUsername", PortalID, GetNull(Username), GetNull(Rolename)), IDataReader)
        End Function

        Public Overrides Function AddUserRole(ByVal PortalId As Integer, ByVal UserId As Integer, ByVal RoleId As Integer, ByVal EffectiveDate As Date, ByVal ExpiryDate As Date) As Integer
            Return CType(SqlHelper.ExecuteScalar(ConnectionString, DatabaseOwner & ObjectQualifier & "AddUserRole", PortalId, UserId, RoleId, GetNull(EffectiveDate), GetNull(ExpiryDate)), Integer)
        End Function

        Public Overrides Sub UpdateUserRole(ByVal UserRoleId As Integer, ByVal EffectiveDate As Date, ByVal ExpiryDate As Date)
            SqlHelper.ExecuteNonQuery(ConnectionString, DatabaseOwner & ObjectQualifier & "UpdateUserRole", UserRoleId, GetNull(EffectiveDate), GetNull(ExpiryDate))
        End Sub

        Public Overrides Sub DeleteUserRole(ByVal UserId As Integer, ByVal RoleId As Integer)
            SqlHelper.ExecuteNonQuery(ConnectionString, DatabaseOwner & ObjectQualifier & "DeleteUserRole", UserId, RoleId)
        End Sub

        Public Overrides Function GetServices(ByVal PortalId As Integer, ByVal UserId As Integer) As IDataReader
            Return CType(SqlHelper.ExecuteReader(ConnectionString, DatabaseOwner & ObjectQualifier & "GetServices", PortalId, GetNull(UserId)), IDataReader)
        End Function

        Public Overrides Function GetUsers(ByVal PortalId As Integer) As IDataReader
            Return CType(SqlHelper.ExecuteReader(ConnectionString, DatabaseOwner & ObjectQualifier & "GetUsers", GetNull(PortalId)), IDataReader)
        End Function

#End Region

    End Class

End Namespace