'
' DotNetNuke - http://www.dotnetnuke.com
' Copyright (c) 2002-2006
' by Perpetual Motion Interactive Systems Inc. ( http://www.perpetualmotion.ca )
'
' Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
' documentation files (the "Software"), to deal in the Software without restriction, including without limitation 
' the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and 
' to permit persons to whom the Software is furnished to do so, subject to the following conditions:
'
' The above copyright notice and this permission notice shall be included in all copies or substantial portions 
' of the Software.
'
' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED 
' TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
' THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
' CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
' DEALINGS IN THE SOFTWARE.
'
Imports DotNetNuke.Entities.Modules
Imports DotNetNuke.Entities.Modules.Actions
Imports DotNetNuke.Entities.Profile

Imports DotNetNuke.Services.Localization

Imports DotNetNuke.UI.Utilities
Imports DotNetNuke.UI.WebControls

Namespace DotNetNuke.Modules.Admin.Users

	''' -----------------------------------------------------------------------------
	''' <summary>
    ''' The EditProfileDefinition PortalModuleBase is used to manage a Profile Property
    ''' for a portal
	''' </summary>
    ''' <remarks>
	''' </remarks>
	''' <history>
    ''' 	[cnurse]	02/22/2006  Created
    ''' </history>
	''' -----------------------------------------------------------------------------
    Partial Class EditProfileDefinition
        Inherits Entities.Modules.PortalModuleBase

#Region "Private Members"

        Private PropertyDefinitionId As Integer = Null.NullInteger
#End Region

#Region "Protected Members"

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Gets whether we are dealing with SuperUsers
        ''' </summary>
        ''' <history>
        ''' 	[cnurse]	05/11/2006  Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Protected ReadOnly Property IsSuperUser() As Boolean
            Get
                If PortalSettings.ActiveTab.ParentId = PortalSettings.SuperTabId Then
                    Return True
                Else
                    Return False
                End If
            End Get
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Gets the Portal Id whose Users we are managing
        ''' </summary>
        ''' <history>
        ''' 	[cnurse]	05/11/2006  Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Protected ReadOnly Property UsersPortalId() As Integer
            Get
                Dim intPortalId As Integer = PortalId
                If IsSuperUser Then
                    intPortalId = Null.NullInteger
                End If
                Return intPortalId
            End Get
        End Property

#End Region

#Region "Event Handlers"

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Page_Init runs when the control is initialised
        ''' </summary>
        ''' <history>
        ''' 	[cnurse]	02/22/2006  Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Private Sub Page_Init(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Init
            'Get Property Definition Id from Querystring
            If Not (Request.QueryString("PropertyDefinitionId") Is Nothing) Then
                PropertyDefinitionId = Int32.Parse(Request.QueryString("PropertyDefinitionId"))
            End If

        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Page_Load runs when the control is loaded
        ''' </summary>
        ''' <history>
        ''' 	[cnurse]	02/22/2006  Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Private Sub Page_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
            Try
                'Declare Property Definition object
                Dim propertyDefinition As ProfilePropertyDefinition

                If PropertyDefinitionId <> Null.NullInteger Then
                    'Get Property Definition from Data Store
                    propertyDefinition = ProfileController.GetPropertyDefinition(PropertyDefinitionId)
                Else
                    'Create New Property Definition
                    propertyDefinition = New ProfilePropertyDefinition
                    propertyDefinition.PortalId = PortalId
                End If

                cmdDelete.Visible = True
                ClientAPI.AddButtonConfirm(cmdDelete, Localization.GetString("DeleteItem"))

                'Bind Property Definition to Data Store
                Properties.LocalResourceFile = Me.LocalResourceFile
                Properties.DataSource = propertyDefinition
                Properties.DataBind()

            Catch exc As Exception    'Module failed to load
                ProcessModuleLoadException(Me, exc)
            End Try
        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' cmdCancel_Click runs when the Cancel button is clciked
        ''' </summary>
        ''' <history>
        ''' 	[cnurse]	02/22/2006  Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Private Sub cmdCancel_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles cmdCancel.Click
            Try
                'Redirect to Definitions page
                Response.Redirect(NavigateURL(TabId, "ManageProfile", "mid=" & ModuleId), True)
            Catch exc As Exception    'Module failed to load
                ProcessModuleLoadException(Me, exc)
            End Try
        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' cmdDelete_Click runs when the Delete button is clciked
        ''' </summary>
        ''' <history>
        ''' 	[cnurse]	02/22/2006  Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Private Sub cmdDelete_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles cmdDelete.Click
            Try
                If PropertyDefinitionId <> Null.NullInteger Then
                    'Declare Definition and "retrieve" it from the Property Editor
                    Dim propertyDefinition As ProfilePropertyDefinition
                    propertyDefinition = DirectCast(Properties.DataSource, ProfilePropertyDefinition)

                    'Delete the Property Definition
                    ProfileController.DeletePropertyDefinition(propertyDefinition)
                End If

                'Clear Profile Definition Cache
                ProfileController.ClearProfileDefinitionCache(UsersPortalId)

                'Redirect to Definitions page
                Response.Redirect(NavigateURL(TabId, "ManageProfile", "mid=" & ModuleId), True)
            Catch exc As Exception    'Module failed to load
                ProcessModuleLoadException(Me, exc)
            End Try
        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' cmdUpdate_Click runs when the Update button is clciked
        ''' </summary>
        ''' <history>
        ''' 	[cnurse]	02/22/2006  Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Private Sub cmdUpdate_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles cmdUpdate.Click
            Try
                'Check if Property Editor has been updated by user
                If Properties.IsDirty Then
                    'Declare Definition and "retrieve" it from the Property Editor
                    Dim propertyDefinition As ProfilePropertyDefinition
                    propertyDefinition = DirectCast(Properties.DataSource, ProfilePropertyDefinition)

                    If PropertyDefinitionId = Null.NullInteger Then
                        'Add the Property Definition
                        PropertyDefinitionId = ProfileController.AddPropertyDefinition(propertyDefinition)

                        If PropertyDefinitionId < Null.NullInteger Then
                            UI.Skins.Skin.AddModuleMessage(Me, Localization.GetString("DuplicateName", Me.LocalResourceFile), Skins.Controls.ModuleMessage.ModuleMessageType.RedError)
                        End If
                    Else
                        'Update the Property Definition
                        ProfileController.UpdatePropertyDefinition(propertyDefinition)
                    End If
                End If

                'Clear Profile Definition Cache
                ProfileController.ClearProfileDefinitionCache(PortalId)

                'Redirect to Definitions page
                If PropertyDefinitionId > Null.NullInteger Then
                    Response.Redirect(NavigateURL(TabId, "ManageProfile", "mid=" & ModuleId), True)
                End If
            Catch exc As Exception    'Module failed to load
                ProcessModuleLoadException(Me, exc)
            End Try
        End Sub

#End Region

    End Class

End Namespace
